﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Windows.Forms;
using HeuristicLab.Clients.Hive.Administration;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.Services.Hive.Common.DataTransfer;

namespace HeuristicLab.Clients.Hive.Views.Administration {
  [View("HiveAdministrationView")]
  [Content(typeof(HiveAdministrationClient), IsDefaultView = true)]
  public partial class HiveAdministrationView : ItemView {
    public new HiveAdministrationClient Content {
      get { return (HiveAdministrationClient)base.Content; }
      set { base.Content = value; }
    }

    public HiveAdministrationView() {
      InitializeComponent();
    }

    #region Register Content Events
    protected override void DeregisterContentEvents() {
      // TODO: Deregister your event handlers on the Content here
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // TODO: Register your event handlers on the Content here
    }
    #endregion


    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        // TODO: Put code here when content is null
      } else {
        updateSlaveGroups.UpdateAction = new Action(Content.UpdateSlaveGroups);
        treeSlaveGroup.ImageList.Images.Add(HiveImageLibrary.Slave);
        treeSlaveGroup.ImageList.Images.Add(HiveImageLibrary.SlaveGroup);

        foreach (SlaveGroup g in Content.SlaveGroups) {
          TreeNode tn = new TreeNode();
          tn.ImageIndex = treeSlaveGroup.ImageList.Images.Count - 1;
          tn.SelectedImageIndex = tn.ImageIndex;

          tn.Tag = g;
          tn.Text = g.Name;
          foreach (Slave s in Content.Slaves.FindAll(s => s.ParentResourceId != null && s.ParentResourceId == g.Id)) {
            var stn = new TreeNode(s.Name);
            stn.ImageIndex = 0;
            stn.SelectedImageIndex = stn.ImageIndex;
            stn.Tag = s;
            tn.Nodes.Add(stn);
          }
          treeSlaveGroup.Nodes.Add(tn);
        }

        TreeNode ungrp = new TreeNode("UNGROUPED");
        ungrp.ImageIndex = treeSlaveGroup.ImageList.Images.Count - 1;
        ungrp.SelectedImageIndex = ungrp.ImageIndex;
        var newGroup = new SlaveGroup();
        newGroup.Name = "UNGROUPED";
        newGroup.Id = Guid.NewGuid();
        newGroup.Description = "Contains slaves which are in no group";
        ungrp.Tag = newGroup;

        foreach (Slave s in Content.Slaves.FindAll(s => s.ParentResourceId == null)) {
          var stn = new TreeNode(s.Name);
          stn.ImageIndex = 0;
          stn.SelectedImageIndex = stn.ImageIndex;
          stn.Tag = s;
          ungrp.Nodes.Add(stn);
        }
        treeSlaveGroup.Nodes.Add(ungrp);

        ShowUsers();
      }
    }

    private void ShowUsers() {
      lstUsers.SmallImageList.Images.Add(HeuristicLab.Common.Resources.VSImageLibrary.User);

      foreach (string user in Content.Users) {
        ListViewItem lvi = new ListViewItem();
        lvi.Tag = user;
        lvi.Text = user;
        lvi.ImageIndex = 0;
        lstUsers.Items.Add(lvi);
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      // TODO: Put code here to enable or disable controls based on whether the Content is/not null or the view is ReadOnly
    }

    private void HiveAdministrationView_Load(object sender, System.EventArgs e) {
      if (Content != null) {

      }
    }

    private void treeSlaveGroup_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e) {
      if (e.Node != null) {
        if (e.Node.Tag.GetType() == typeof(Slave)) {
          Slave s = e.Node.Tag as Slave;
          txtName.Text = s.Name;
          txtDetailsDescription.Text = s.Description;
          txtCPU.Text = s.CpuArchitecture + ", " + s.Cores + " Cores @ " + s.CpuSpeed.ToString();
          txtMemory.Text = s.Memory.ToString();
          txtDetailsCalculatedJobs.Clear();
          txtOS.Text = s.OperatingSystem;
          txtSlaveState.Text = s.SlaveState.ToString();
        } else if (e.Node.Tag.GetType() == typeof(SlaveGroup)) {
          SlaveGroup s = e.Node.Tag as SlaveGroup;
          txtName.Text = s.Name;
          txtDetailsDescription.Text = s.Description;
          txtCPU.Clear();
          txtMemory.Clear();
          txtDetailsCalculatedJobs.Clear();
          txtOS.Clear();
          txtSlaveState.Clear();
        }
      }
    }


    #region Event Handlers
    // TODO: Put event handlers here
    #endregion
  }
}
